# Copyright (c) 2013 New Dream Network, LLC (DreamHost)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Copyright (C) 2013 Association of Universities for Research in Astronomy
#                    (AURA)
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     1. Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#
#     2. Redistributions in binary form must reproduce the above
#        copyright notice, this list of conditions and the following
#        disclaimer in the documentation and/or other materials provided
#        with the distribution.
#
#     3. The name of AURA and its representatives may not be used to
#        endorse or promote products derived from this software without
#        specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY AURA ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL AURA BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS

import testtools

from pbr._compat import easy_install
from pbr._compat import metadata


class TestPackagingHelpers(testtools.TestCase):

    def test_generate_script(self):
        group = 'console_scripts'
        entry_point = metadata.entrypoint(
            module_name='pbr.packaging',
            attrs=('LocalInstallScripts',),
        )
        header = '#!/usr/bin/env fake-header\n'
        template = (
            '%(group)s %(module_name)s %(import_target)s %(invoke_target)s'
        )

        generated_script = easy_install.generate_script(
            group, entry_point, header, template
        )

        expected_script = (
            '#!/usr/bin/env fake-header\nconsole_scripts pbr.packaging '
            'LocalInstallScripts LocalInstallScripts'
        )
        self.assertEqual(expected_script, generated_script)

    def test_generate_script_validates_expectations(self):
        group = 'console_scripts'
        entry_point = metadata.entrypoint(
            module_name='pbr.packaging',
            attrs=None,
        )
        header = '#!/usr/bin/env fake-header\n'
        template = (
            '%(group)s %(module_name)s %(import_target)s %(invoke_target)s'
        )
        self.assertRaises(
            ValueError,
            easy_install.generate_script,
            group,
            entry_point,
            header,
            template,
        )

        entry_point = metadata.entrypoint(
            module_name='pbr.packaging',
            attrs=('attr1', 'attr2', 'attr3'),
        )
        self.assertRaises(
            ValueError,
            easy_install.generate_script,
            group,
            entry_point,
            header,
            template,
        )
